##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved
#
# Name:
#    ixia_dhcp_api.tcl
#
# Purpose:
#    A script development library containing DHCP APIs for test automation with 
#    the Ixia chassis.
#
# Author:
#    Ixia engineering; direct all communication to support@ixiacom.com
#
# Usage:
#
# Description:
#    The procedures contained within this library include:
#
#    -emulation_dhcp_config 
#    -emulation_dhcp_group_config 
#    -emulation_dhcp_control 
#    -emulation_dhcp_stats
#
# Requirements:
#    ixiaapiutils.tcl, a library containing TCL utilities 
#    parseddashedargs.tcl, a library containing the procDescr
#
# Variables:
# 
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the users requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################


proc ::ixia::emulation_dhcp_config { args } {

    variable executeOnTclServer
      
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
	
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_dhcp_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable dhcp_handles_array
	
    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -mode                        CHOICES create modify reset
                                     DEFAULT create
    }
    
    set opt_args {        
        -accept_partial_config          CHOICES 0 1
                                        DEFAULT 0
        -handle                         ANY
        -lease_time                     RANGE 0-2147483647
                                        DEFAULT 3600
        -max_dhcp_msg_size              RANGE 0-65535
                                        DEFAULT 576
        -mode                           CHOICES create modify reset
                                        DEFAULT create
        -msg_timeout                    NUMERIC
                                        DEFAULT 4
        -outstanding_releases_count     RANGE 1-100000
                                        DEFAULT 500
        -outstanding_session_count      NUMERIC
                                        DEFAULT 50
        -port_handle                    REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -release_rate                   RANGE 1-100000
                                        DEFAULT 50
        -release_rate_increment         RANGE 0-100000
                                        DEFAULT 50
        -request_rate                   RANGE 1-100000
                                        DEFAULT 10
        -request_rate_increment         RANGE 0-100000
                                        DEFAULT 50
        -retry_count                    RANGE 1-100
                                        DEFAULT 3
        -server_port                    RANGE 0-65535
                                        DEFAULT 67
        -wait_for_completion            CHOICES 0 1
                                        DEFAULT 0
        -associates                     ANY
        -dhcp6_echo_ia_info             CHOICES 0 1
                                        DEFAULT 0
        -dhcp6_reb_max_rt               RANGE 1-10000
                                        DEFAULT 600
        -dhcp6_reb_timeout              RANGE 1-100
                                        DEFAULT 10
        -dhcp6_rel_max_rc               RANGE 1-100
                                        DEFAULT 5
        -dhcp6_rel_timeout              RANGE 1-100
                                        DEFAULT 1
        -dhcp6_ren_max_rt               RANGE 1-10000
                                        DEFAULT 600
        -dhcp6_ren_timeout              RANGE 1-100
                                        DEFAULT 10
        -dhcp6_req_max_rc               RANGE 1-100
                                        DEFAULT 5
        -dhcp6_req_max_rt               RANGE 1-10000
                                        DEFAULT 30
        -dhcp6_req_timeout              RANGE 1-100
                                        DEFAULT 1
        -dhcp6_sol_max_rc               RANGE 1-100
                                        DEFAULT 3
        -dhcp6_sol_max_rt               RANGE 1-10000
                                        DEFAULT 120
        -dhcp6_sol_timeout              RANGE 1-100
                                        DEFAULT 1
        -msg_timeout_factor             RANGE 1-100
                                        DEFAULT 2
        -no_write                       FLAG
        -override_global_setup_rate     CHOICES 0 1
                                        DEFAULT 1
        -override_global_teardown_rate  CHOICES 0 1
                                        DEFAULT 1
        -release_rate_max               RANGE 1-100000
                                        DEFAULT 500
        -request_rate_max               RANGE 1-100000
                                        DEFAULT 50
        -reset                          FLAG
        -version                        CHOICES ixtclhal ixaccess ixnetwork
                                        DEFAULT ixtclhal
    }

    if [catch {::ixia::parse_dashed_args -args $args -optional_args $opt_args \
            -mandatory_args $man_args} errorMsg] {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: $errorMsg."
        return $returnList
    }

    # Check $port_handle count
    
    if {[info exists port_handle] && 1 != [llength $port_handle]} {\
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: Please provide a single port handle"
        return $returnList
    }

    # Check for ixAccess|IxTclHal|IxNetwork version
    if {$version == "ixaccess"} {    
        if {[catch {package present IxTclAccess} versionIxTclAccess] || \
                    ($versionIxTclAccess < 2.20)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: DHCP cannot\
                    be configured on this IxAccess version."
            return $returnList
        }
    } elseif {$version == "ixtclhal"} {
        if {[catch {package present IxTclHal} versionIxTclHal] || \
                ($versionIxTclHal < 4.00)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: DHCP cannot\
                    be configured on this IxOS version."
            return $returnList
        }
    } else {
        if {[catch {package present IxTclNetwork} versionIxNetwork] || \
                ($versionIxNetwork < 5.30)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: IxNetwork version not supported. \
                Please upgrade."
            return $returnList
        }
        return [::ixia::ixnetwork_dhcp_config $args $opt_args $man_args]        
    }
    
    # here we decide if we use ixaccess or ixtclhal, depending on -version value
    if {$version == "ixaccess"} {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: \
                IxAccess API is not supported"
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
    
    return $returnList
}


proc ::ixia::emulation_dhcp_group_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_dhcp_group_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable dhcp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -mode                                          CHOICES create modify reset
                                                       DEFAULT create
        -handle                                        ANY    
    }
    
    set opt_args {
        -encap                                         CHOICES ethernet_ii ethernet_ii_vlan ethernet_ii_qinq 
                                                       CHOICES vc_mux_ipv4_routed vc_mux_fcs vc_mux vc_mux_ipv6_routed llcsnap_routed llcsnap_fcs llcsnap llcsnap_ppp vc_mux_ppp 
        -mac_addr                                      MAC
        -mac_addr_step                                 MAC
                                                       DEFAULT 00.00.00.00.00.01
        -num_sessions                                  RANGE 1-65536
        -pvc_incr_mode                                 CHOICES vci vpi pvc
                                                       DEFAULT vci
        -qinq_incr_mode                                CHOICES inner outer both 
                                                       DEFAULT inner
        -sessions_per_vc                               RANGE 1-65535
                                                       DEFAULT 1
        -vci                                           RANGE 0-65535
        -vci_count                                     RANGE 0-65535
        -vci_step                                      NUMERIC
        -vlan_id                                       RANGE 0-4095
                                                       DEFAULT 4094
        -vlan_id_count                                 RANGE 0-4095
                                                       DEFAULT 4094
        -vlan_id_outer                                 NUMERIC
        -vlan_id_outer_count                           RANGE 1-4094
        -vlan_id_outer_step                            RANGE 1-4094
        -vlan_id_step                                  RANGE 0-4095
        -vpi                                           RANGE 0-255
        -vpi_count                                     RANGE 0-255
        -vpi_step                                      NUMERIC
        -dhcp6_range_duid_enterprise_id                NUMERIC
                                                       DEFAULT 10
        -dhcp6_range_duid_type                         CHOICES duid_llt duid_en duid_ll
                                                       DEFAULT duid_llt
        -dhcp6_range_duid_vendor_id                    NUMERIC
                                                       DEFAULT 10
        -dhcp6_range_duid_vendor_id_increment          NUMERIC
                                                       DEFAULT 1
        -dhcp6_range_ia_id                             NUMERIC
                                                       DEFAULT 10
        -dhcp6_range_ia_id_increment                   NUMERIC
                                                       DEFAULT 1
        -dhcp6_range_ia_t1                             NUMERIC
                                                       DEFAULT 302400
        -dhcp6_range_ia_t2                             NUMERIC
                                                       DEFAULT 483840
        -dhcp6_range_ia_type                           CHOICES iana iata iapd iana_iapd
                                                       DEFAULT iana
        -dhcp6_range_param_request_list                RANGE 2-24
        -dhcp_range_ip_type                            CHOICES ipv4 ipv6
                                                       DEFAULT ipv4
        -dhcp_range_param_request_list                 RANGE 1-90
        -dhcp_range_relay6_hosts_per_opt_interface_id  RANGE 1-100
                                                       DEFAULT 1
        -dhcp_range_relay6_opt_interface_id            ANY
                                                       DEFAULT "id-\[001-900\]"
        -dhcp_range_relay6_use_opt_interface_id        CHOICES 0 1
                                                       DEFAULT 0
        -dhcp_range_relay_address_increment            IP
                                                       DEFAULT 0.0.0.1
        -dhcp_range_relay_circuit_id                   ANY
                                                       DEFAULT CIRCUITID-p
        -dhcp_range_relay_count                        RANGE 1-1000000
                                                       DEFAULT 1
        -dhcp_range_relay_destination                  IP
                                                       DEFAULT 20.0.0.1
        -dhcp_range_relay_first_address                IP
                                                       DEFAULT 20.0.0.100
        -dhcp_range_relay_first_vlan_id                RANGE 1-4094
                                                       DEFAULT 1
        -dhcp_range_relay_gateway                      IP
                                                       DEFAULT 20.0.0.1
        -dhcp_range_relay_hosts_per_circuit_id         RANGE 1-100
                                                       DEFAULT 1
        -dhcp_range_relay_hosts_per_remote_id          RANGE 1-100
                                                       DEFAULT 1
        -dhcp_range_relay_override_vlan_settings       CHOICES 0 1
                                                       DEFAULT 0
        -dhcp_range_relay_remote_id                    ANY
                                                       DEFAULT REMOTEID-I
        -dhcp_range_relay_subnet                       RANGE 1-128
                                                       DEFAULT 24
        -dhcp_range_relay_use_circuit_id               CHOICES 0 1
                                                       DEFAULT 0
        -dhcp_range_relay_use_remote_id                CHOICES 0 1
                                                       DEFAULT 0
        -dhcp_range_relay_use_suboption6               CHOICES 0 1
                                                       DEFAULT 0
        -dhcp_range_relay_vlan_count                   RANGE 1-4094
                                                       DEFAULT 1
        -dhcp_range_relay_vlan_increment               RANGE 0-4093
                                                       DEFAULT 1
        -dhcp_range_renew_timer                        NUMERIC
                                                       DEFAULT 0
        -dhcp_range_server_address                     IP
                                                       DEFAULT 10.0.0.1
        -dhcp_range_suboption6_address_subnet          RANGE 1-32
                                                       DEFAULT 24
        -dhcp_range_suboption6_first_address           IP
                                                       DEFAULT 20.1.1.100
        -dhcp_range_use_first_server                   CHOICES 0 1
                                                       DEFAULT 1
        -dhcp_range_use_relay_agent                    CHOICES 0 1
                                                       DEFAULT 0
        -dhcp_range_use_trusted_network_element        CHOICES 0 1
                                                       DEFAULT 0
        -mac_mtu                                       RANGE 500-9500
                                                       DEFAULT 1500
        -no_write                                      FLAG
        -server_id                                     IP
        -target_subport                                RANGE 0-3
                                                       DEFAULT 0
        -use_vendor_id                                 CHOICES 0 1
                                                       DEFAULT 0
        -vendor_id                                     ANY
                                                       DEFAULT Ixia
        -version                                       CHOICES ixtclhal ixaccess ixnetwork
                                                       DEFAULT ixtclhal
        -vlan_id_outer_increment_step                  RANGE 0-4093
        -vlan_id_increment_step                        RANGE 0-4093
        -vlan_id_outer_priority                        RANGE 0-7
                                                       DEFAULT 0
        -vlan_user_priority                            RANGE 0-7
                                                       DEFAULT 0
    }

    if [catch {::ixia::parse_dashed_args -args $args -optional_args $opt_args \
            -mandatory_args $man_args} errorMsg] {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: $errorMsg."
        return $returnList
    }
    if {![info exists encap]} {
        set encap ethernet_ii
    }
    if {[info exists mac_addr_step]} {
        if {$version == "ixaccess"} {
            if {[regexp -- {\d+\.\d+} $mac_addr_step] != 0} {
                set mac_addr_step [mac2num $mac_addr_step]
            }
        } else {
            if {[regexp -- {^\d+$} $mac_addr_step] != 0} {
                set mac_addr_step [num2mac $mac_addr_step]
            }
        }
    }
    # Check for ixAccess|IxTclHal version
    if {$version == "ixaccess"} {
        if {[catch {package present IxTclAccess} versionIxTclAccess] || \
                    ($versionIxTclAccess < 2.20)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: DHCP cannot\
                    be configured on this IxAccess version."
            return $returnList
        }
        set encapList [list vc_mux llcsnap]
        if {[info exists encap] && ([lsearch $encapList $encap] == 0)} {
           keylset returnList status $::FAILURE
           keylset returnList log "ERROR in $procName: This encap ($encap)\
                   is not available when using ixaccess version."
           return $returnList
        }

        set returnList [::ixia::ixa_dhcp_group_config $args]
        return $returnList
    } elseif {$version == "ixtclhal"} {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
    } else {
        # ixnetwork implementez-moi
        if {[catch {package present IxTclNetwork} versionIxNetwork] || \
                ($versionIxNetwork < 5.30)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: IxNetwork version not supported.\
                Please upgrade."
            return $returnList
        }
        return [::ixia::ixnetwork_dhcp_group_config $args $opt_args $man_args]        
    }


    return $returnList
}


proc ::ixia::emulation_dhcp_control { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_dhcp_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable dhcp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -action        CHOICES abort abort_async bind release renew
                       DEFAULT bind
    }
    
    set opt_args {
        -port_handle   ANY
        -handle        ANY
        -no_write      FLAG
        -request_rate  RANGE 0-4294967295
                       DEFAULT 100    
    }
    
     if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    if {[catch {::ixia::parse_dashed_args -args $args -optional_args $opt_args \
            -mandatory_args $man_args} errorMsg]} {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: $errorMsg"
        return $returnList
    }
    
    # Check for ixnetwork request
    if {$::ixia::new_ixnetwork_api == 0 && $::ixia::no_more_tclhal == 0} {
        set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
    } elseif {[catch {package present IxTclNetwork} versionIxNetwork]} {
        #
    } elseif {($versionIxNetwork < 5.30)} {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: IxNetwork version not supported.\
                Please upgrade."
        return $returnList
    } else {
        return [::ixia::ixnetwork_dhcp_group_control $args $opt_args $man_args]
    }
    
    return $returnList
}


proc ::ixia::emulation_dhcp_stats { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_dhcp_stats $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable dhcp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    set opt_args {
        -port_handle  ANY
        -action       CHOICES clear
        -handle       ANY
        -mode         CHOICES session aggregate_stats
        -no_write     FLAG
        -version      CHOICES ixtclhal ixaccess ixnetwork
                      DEFAULT ixtclhal
    }
    
    if [catch {::ixia::parse_dashed_args -args $args -optional_args $opt_args} errorMsg] {
        keylset returnList status $::FAILURE
        keylset returnList log "ERROR in $procName: $errorMsg."
        return $returnList
    }
    
    # Check for ixAccess|IxTclHal version
    if {$version == "ixaccess"} {    
        if {[catch {package present IxTclAccess} versionIxTclAccess] || \
                    ($versionIxTclAccess < 2.20)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: DHCP cannot\
                    be configured on this IxAccess version."
            return $returnList
        }
        set returnList [::ixia::ixa_dhcp_stats $args]
        return $returnList
    } elseif {$version == "ixtclhal"} {
        set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
    } else {
        if {[catch {package present IxTclNetwork} versionIxNetwork] || \
                ($versionIxNetwork < 5.30)} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: IxNetwork version not supported.\
                Please upgrade."
            return $returnList
        }
        return [::ixia::ixnetwork_dhcp_group_stats $args $opt_args]
    }
    
    return $returnList
}
